import { Component, HostBinding, Input, NgModule, ViewChild } from "@angular/core";
import { NG_VALUE_ACCESSOR } from "@angular/forms";
import { IgxRippleModule } from "../directives/ripple/ripple.directive";
export var LabelPosition;
(function (LabelPosition) {
    LabelPosition["BEFORE"] = "before";
    LabelPosition["AFTER"] = "after";
})(LabelPosition || (LabelPosition = {}));
var noop = function () { };
var ɵ0 = noop;
var nextId = 0;
var IgxCheckboxComponent = /** @class */ (function () {
    function IgxCheckboxComponent() {
        this.focused = false;
        this.id = "igx-checkbox-" + nextId++;
        this.tabindex = null;
        this.labelPosition = LabelPosition.AFTER;
        this.disableRipple = false;
        this.cssClass = "igx-checkbox";
        this.indeterminate = false;
        this.checked = false;
        this.disabled = false;
        this._onTouchedCallback = noop;
        this._onChangeCallback = noop;
    }
    IgxCheckboxComponent.prototype.onChange = function (event) {
        if (this.disabled) {
            return;
        }
        this.indeterminate = false;
        this.checked = !this.checked;
        this._onChangeCallback(this.checked);
    };
    IgxCheckboxComponent.prototype.onFocus = function (event) {
        this.focused = true;
    };
    IgxCheckboxComponent.prototype.onBlur = function (event) {
        this.focused = false;
        this._onTouchedCallback();
    };
    IgxCheckboxComponent.prototype.writeValue = function (value) {
        if (this.disabled) {
            return;
        }
        this._value = value;
        this.checked = !!this._value;
    };
    Object.defineProperty(IgxCheckboxComponent.prototype, "labelClass", {
        get: function () {
            switch (this.labelPosition) {
                case LabelPosition.BEFORE:
                    return this.cssClass + "__label--before";
                case LabelPosition.AFTER:
                default:
                    return this.cssClass + "__label";
            }
        },
        enumerable: true,
        configurable: true
    });
    IgxCheckboxComponent.prototype.registerOnChange = function (fn) { this._onChangeCallback = fn; };
    IgxCheckboxComponent.prototype.registerOnTouched = function (fn) { this._onTouchedCallback = fn; };
    IgxCheckboxComponent.decorators = [
        { type: Component, args: [{
                    providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: IgxCheckboxComponent, multi: true }],
                    selector: "igx-checkbox",
                    preserveWhitespaces: false,
                    template: "<input class=\"igx-checkbox__input\" #checkbox type=\"checkbox\" [attr.id]=\"id\" [attr.name]=\"name\" [attr.value]=\"value\" [attr.tabindex]=\"tabindex\"     [attr.disabled]=\"disabled ? true : null\" [indeterminate]=\"indeterminate\" [checked]=\"checked\" [attr.aria-checked]=\"checked\"     (change)=\"onChange($event)\" (focus)=\"onFocus($event)\" (blur)=\"onBlur($event)\">  <label class=\"igx-checkbox__composite\" igxRipple [igxRippleDisabled]=\"disableRipple\" [igxRippleCentered]=\"true\" [igxRippleDuration]=\"100\"     igxRippleTarget=\".igx-checkbox__ripple\" [attr.for]=\"id\">     <svg class=\"igx-checkbox__composite-mark\" xmlns=\"http://www.w3.org/2000/svg\" viewBox=\"0 0 24 24\">         <path d=\"M4.1,12.7 9,17.6 20.3,6.3\" />     </svg>     <div class=\"igx-checkbox__ripple\"></div> </label>  <span [class]=\"labelClass\" (click)=\"onChange($event)\">     <ng-content></ng-content> </span>"
                },] },
    ];
    /** @nocollapse */
    IgxCheckboxComponent.ctorParameters = function () { return []; };
    IgxCheckboxComponent.propDecorators = {
        "nativeCheckbox": [{ type: ViewChild, args: ["checkbox",] },],
        "id": [{ type: Input },],
        "value": [{ type: Input },],
        "name": [{ type: Input },],
        "tabindex": [{ type: Input },],
        "labelPosition": [{ type: Input },],
        "disableRipple": [{ type: Input },],
        "cssClass": [{ type: HostBinding, args: ["class.igx-checkbox",] },],
        "indeterminate": [{ type: HostBinding, args: ["class.igx-checkbox--indeterminate",] }, { type: Input },],
        "checked": [{ type: HostBinding, args: ["class.igx-checkbox--checked",] }, { type: Input },],
        "disabled": [{ type: HostBinding, args: ["class.igx-checkbox--disabled",] }, { type: Input },],
    };
    return IgxCheckboxComponent;
}());
export { IgxCheckboxComponent };
var IgxCheckboxModule = /** @class */ (function () {
    function IgxCheckboxModule() {
    }
    IgxCheckboxModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [IgxCheckboxComponent],
                    exports: [IgxCheckboxComponent],
                    imports: [IgxRippleModule]
                },] },
    ];
    /** @nocollapse */
    IgxCheckboxModule.ctorParameters = function () { return []; };
    return IgxCheckboxModule;
}());
export { IgxCheckboxModule };
export { ɵ0 };
//# sourceMappingURL=checkbox.component.js.map