import { ComponentFactoryResolver, ComponentRef, EventEmitter, OnDestroy, OnInit, ViewContainerRef } from "@angular/core";
import { ControlValueAccessor } from "@angular/forms";
import { IgxCalendarComponent, IgxCalendarHeaderTemplateDirective, IgxCalendarSubheaderTemplateDirective, WEEKDAYS } from "../calendar";
import { IgxDialogComponent } from "../dialog/dialog.component";
export declare class IgxDatePickerComponent implements ControlValueAccessor, OnInit, OnDestroy {
    private resolver;
    formatter: (val: Date) => string;
    isDisabled: boolean;
    value: Date;
    /**
     * Propagate calendar properties.
     */
    locale: string;
    weekStart: WEEKDAYS | number;
    formatOptions: {
        day: string;
        month: string;
        weekday: string;
        year: string;
    };
    formatViews: {
        day: boolean;
        month: boolean;
        year: boolean;
    };
    vertical: boolean;
    /**
     * Propagate dialog properties.
     */
    todayButtonLabel: string;
    cancelButtonLabel: string;
    onOpen: EventEmitter<{}>;
    /**
     * Propagate clanedar events.
     */
    onSelection: EventEmitter<Date>;
    readonly displayData: string;
    headerTemplate: IgxCalendarHeaderTemplateDirective;
    subheaderTemplate: IgxCalendarSubheaderTemplateDirective;
    container: ViewContainerRef;
    alert: IgxDialogComponent;
    calendarRef: ComponentRef<IgxCalendarComponent>;
    readonly calendar: IgxCalendarComponent;
    constructor(resolver: ComponentFactoryResolver);
    writeValue(value: Date): void;
    registerOnChange(fn: (_: Date) => void): void;
    registerOnTouched(fn: () => void): void;
    ngOnInit(): void;
    ngOnDestroy(): void;
    /**
     * Selects today's date from calendar and change the input field value, @calendar.viewDate and @calendar.value.
     */
    triggerTodaySelection(): void;
    /**
     * Change the calendar slection and calling this method will emit the @calendar.onSelection event,
     * which will fire @handleSelection method.
     * @param date passed date that has to be set to the calendar.
     */
    selectDate(date: Date): void;
    /**
     * Emits the open event and focus the dialog.
     */
    onOpenEvent(event: any): void;
    /**
     * Closes the dialog, after was clearing all calendar items from dom.
     */
    handleDialogCloseAction(): void;
    /**
     * Evaluates when @calendar.onSelection event was fired
     * and update the input value.
     *
     * @param event selected value from calendar.
     */
    handleSelection(event: any): void;
    private updateCalendarInstance();
    private _focusTheDialog();
    private _setLocaleToDate(value, locale?);
    /**
     * Apply custom user formatter upon date.
     * @param formatter custom formatter function.
     * @param date passed date
     */
    private _customFormatChecker(formatter, date);
    private _onTouchedCallback;
    private _onChangeCallback;
}
export declare class IgxDatePickerModule {
}
