import { CommonModule } from "@angular/common";
import { Component, ComponentFactoryResolver, ContentChild, EventEmitter, Input, NgModule, Output, ViewChild, ViewContainerRef } from "@angular/core";
import { NG_VALUE_ACCESSOR } from "@angular/forms";
import { IgxCalendarComponent, IgxCalendarHeaderTemplateDirective, IgxCalendarModule, IgxCalendarSubheaderTemplateDirective, WEEKDAYS } from "../calendar";
import { IgxDialogComponent, IgxDialogModule } from "../dialog/dialog.component";
import { IgxInputModule } from "../directives/input/input.directive";
var IgxDatePickerComponent = /** @class */ (function () {
    function IgxDatePickerComponent(resolver) {
        this.resolver = resolver;
        /**
             * Propagate calendar properties.
             */
        this.locale = Constants.DEFAULT_LOCALE_DATE;
        this.weekStart = WEEKDAYS.SUNDAY;
        this.formatOptions = {
            day: "numeric",
            month: "short",
            weekday: "short",
            year: "numeric"
        };
        this.formatViews = {
            day: false,
            month: true,
            year: false
        };
        this.vertical = false;
        this.onOpen = new EventEmitter();
        /**
             * Propagate clanedar events.
             */
        this.onSelection = new EventEmitter();
        this._onTouchedCallback = function () { };
        this._onChangeCallback = function () { };
    }
    Object.defineProperty(IgxDatePickerComponent.prototype, "displayData", {
        get: function () {
            if (this.value) {
                return this._customFormatChecker(this.formatter, this.value);
            }
            return "";
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxDatePickerComponent.prototype, "calendar", {
        get: function () {
            return this.calendarRef.instance;
        },
        enumerable: true,
        configurable: true
    });
    IgxDatePickerComponent.prototype.writeValue = function (value) {
        this.value = value;
    };
    IgxDatePickerComponent.prototype.registerOnChange = function (fn) { this._onChangeCallback = fn; };
    IgxDatePickerComponent.prototype.registerOnTouched = function (fn) { this._onTouchedCallback = fn; };
    IgxDatePickerComponent.prototype.ngOnInit = function () {
        var _this = this;
        /**
                 * If we have passed value from user, update @calendar.value and @calendar.viewDate.
                 */
        this.alert.onOpen.subscribe(function (ev) { return _this._focusTheDialog(); });
        this.alert.onClose.subscribe(function (ev) { return _this.handleDialogCloseAction(); });
    };
    IgxDatePickerComponent.prototype.ngOnDestroy = function () {
        this.alert.onClose.unsubscribe();
        this.alert.onOpen.unsubscribe();
    };
    /**
     * Selects today's date from calendar and change the input field value, @calendar.viewDate and @calendar.value.
     */
    /**
         * Selects today's date from calendar and change the input field value, @calendar.viewDate and @calendar.value.
         */
    IgxDatePickerComponent.prototype.triggerTodaySelection = /**
         * Selects today's date from calendar and change the input field value, @calendar.viewDate and @calendar.value.
         */
    function () {
        var today = new Date(Date.now());
        this.handleSelection(today);
    };
    /**
     * Change the calendar slection and calling this method will emit the @calendar.onSelection event,
     * which will fire @handleSelection method.
     * @param date passed date that has to be set to the calendar.
     */
    /**
         * Change the calendar slection and calling this method will emit the @calendar.onSelection event,
         * which will fire @handleSelection method.
         * @param date passed date that has to be set to the calendar.
         */
    IgxDatePickerComponent.prototype.selectDate = /**
         * Change the calendar slection and calling this method will emit the @calendar.onSelection event,
         * which will fire @handleSelection method.
         * @param date passed date that has to be set to the calendar.
         */
    function (date) {
        this.value = date;
        this.onSelection.emit(date);
    };
    /**
     * Emits the open event and focus the dialog.
     */
    /**
         * Emits the open event and focus the dialog.
         */
    IgxDatePickerComponent.prototype.onOpenEvent = /**
         * Emits the open event and focus the dialog.
         */
    function (event) {
        var factory = this.resolver.resolveComponentFactory(IgxCalendarComponent);
        this.calendarRef = this.container.createComponent(factory);
        this.calendarRef.changeDetectorRef.detach();
        this.updateCalendarInstance();
        this.calendarRef.location.nativeElement.classList.add("igx-date-picker__date--opened");
        this.calendarRef.changeDetectorRef.reattach();
        this.alert.open();
        this._onTouchedCallback();
        this.onOpen.emit(this);
    };
    /**
     * Closes the dialog, after was clearing all calendar items from dom.
     */
    /**
         * Closes the dialog, after was clearing all calendar items from dom.
         */
    IgxDatePickerComponent.prototype.handleDialogCloseAction = /**
         * Closes the dialog, after was clearing all calendar items from dom.
         */
    function () {
        var _this = this;
        setTimeout(function () { return _this.calendarRef.destroy(); }, 350);
    };
    /**
     * Evaluates when @calendar.onSelection event was fired
     * and update the input value.
     *
     * @param event selected value from calendar.
     */
    /**
         * Evaluates when @calendar.onSelection event was fired
         * and update the input value.
         *
         * @param event selected value from calendar.
         */
    IgxDatePickerComponent.prototype.handleSelection = /**
         * Evaluates when @calendar.onSelection event was fired
         * and update the input value.
         *
         * @param event selected value from calendar.
         */
    function (event) {
        this.value = event;
        this.calendar.viewDate = event;
        this._onChangeCallback(event);
        this.alert.close();
        this.onSelection.emit(event);
    };
    IgxDatePickerComponent.prototype.updateCalendarInstance = function () {
        var _this = this;
        this.calendar.formatOptions = this.formatOptions;
        this.calendar.formatViews = this.formatViews;
        this.calendar.locale = this.locale;
        this.calendar.vertical = this.vertical;
        if (this.headerTemplate) {
            this.calendar.headerTemplate = this.headerTemplate;
        }
        if (this.subheaderTemplate) {
            this.calendar.subheaderTemplate = this.subheaderTemplate;
        }
        if (this.value) {
            this.calendar.value = this.value;
            this.calendar.viewDate = this.value;
        }
        this.calendar.weekStart = this.weekStart;
        this.calendar.onSelection.subscribe(function (ev) { return _this.handleSelection(ev); });
    };
    // Focus the dialog element, after its appearence into DOM.
    // Focus the dialog element, after its appearence into DOM.
    IgxDatePickerComponent.prototype._focusTheDialog = 
    // Focus the dialog element, after its appearence into DOM.
    function () {
        var _this = this;
        requestAnimationFrame(function () { return _this.alert.dialogEl.nativeElement.focus(); });
    };
    IgxDatePickerComponent.prototype._setLocaleToDate = function (value, locale) {
        if (locale === void 0) { locale = Constants.DEFAULT_LOCALE_DATE; }
        return value.toLocaleDateString(locale);
    };
    /**
     * Apply custom user formatter upon date.
     * @param formatter custom formatter function.
     * @param date passed date
     */
    /**
         * Apply custom user formatter upon date.
         * @param formatter custom formatter function.
         * @param date passed date
         */
    IgxDatePickerComponent.prototype._customFormatChecker = /**
         * Apply custom user formatter upon date.
         * @param formatter custom formatter function.
         * @param date passed date
         */
    function (formatter, date) {
        return this.formatter ? this.formatter(date) : this._setLocaleToDate(date, this.locale);
    };
    IgxDatePickerComponent.decorators = [
        { type: Component, args: [{
                    providers: [{ provide: NG_VALUE_ACCESSOR, useExisting: IgxDatePickerComponent, multi: true }],
                    // tslint:disable-next-line:component-selector
                    selector: "igx-datePicker",
                    template: "<div [ngClass]=\"{'igx-date-picker': !vertical, 'igx-date-picker--vertical': vertical}\">     <input class=\"igx-date-picker__input-date\" igxInput (click)=\"onOpenEvent($event)\" [value]=\"displayData\" [disabled]=\"isDisabled\"     />     <igx-dialog class=\"igx-date-picker__dialog-popup\" closeOnOutsideSelect=\"true\" [leftButtonLabel]=\"cancelButtonLabel\" (onLeftButtonSelect)=\"alert.close()\"         [rightButtonLabel]=\"todayButtonLabel\" (onRightButtonSelect)=\"triggerTodaySelection()\">         <ng-container #container></ng-container>     </igx-dialog> </div>"
                },] },
    ];
    /** @nocollapse */
    IgxDatePickerComponent.ctorParameters = function () { return [
        { type: ComponentFactoryResolver, },
    ]; };
    IgxDatePickerComponent.propDecorators = {
        "formatter": [{ type: Input },],
        "isDisabled": [{ type: Input },],
        "value": [{ type: Input },],
        "locale": [{ type: Input },],
        "weekStart": [{ type: Input },],
        "formatOptions": [{ type: Input },],
        "formatViews": [{ type: Input },],
        "vertical": [{ type: Input },],
        "todayButtonLabel": [{ type: Input },],
        "cancelButtonLabel": [{ type: Input },],
        "onOpen": [{ type: Output },],
        "onSelection": [{ type: Output },],
        "headerTemplate": [{ type: ContentChild, args: [IgxCalendarHeaderTemplateDirective, { read: IgxCalendarHeaderTemplateDirective },] },],
        "subheaderTemplate": [{ type: ContentChild, args: [IgxCalendarSubheaderTemplateDirective, { read: IgxCalendarSubheaderTemplateDirective },] },],
        "container": [{ type: ViewChild, args: ["container", { read: ViewContainerRef },] },],
        "alert": [{ type: ViewChild, args: [IgxDialogComponent,] },],
    };
    return IgxDatePickerComponent;
}());
export { IgxDatePickerComponent };
var Constants = /** @class */ (function () {
    function Constants() {
    }
    Constants.DEFAULT_LOCALE_DATE = "en";
    return Constants;
}());
var IgxDatePickerModule = /** @class */ (function () {
    function IgxDatePickerModule() {
    }
    IgxDatePickerModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [IgxDatePickerComponent],
                    entryComponents: [IgxCalendarComponent],
                    exports: [IgxDatePickerComponent],
                    imports: [CommonModule, IgxInputModule, IgxDialogModule, IgxCalendarModule]
                },] },
    ];
    /** @nocollapse */
    IgxDatePickerModule.ctorParameters = function () { return []; };
    return IgxDatePickerModule;
}());
export { IgxDatePickerModule };
//# sourceMappingURL=date-picker.component.js.map