import { CommonModule } from "@angular/common";
import { ChangeDetectorRef, ComponentFactoryResolver, Directive, EventEmitter, Input, IterableDiffers, NgModule, NgZone, Output, TemplateRef, ViewChild, ViewContainerRef } from "@angular/core";
import { DisplayContainerComponent } from "./display.container";
import { HVirtualHelperComponent } from "./horizontal.virtual.helper.component";
import { VirtualHelperComponent } from "./virtual.helper.component";
var IgxForOfDirective = /** @class */ (function () {
    function IgxForOfDirective(_viewContainer, _template, _differs, resolver, cdr, _zone) {
        this._viewContainer = _viewContainer;
        this._template = _template;
        this._differs = _differs;
        this.resolver = resolver;
        this.cdr = cdr;
        this._zone = _zone;
        this.state = {
            startIndex: 0,
            chunkSize: 0
        };
        this.igxForRemote = false;
        this.onChunkLoad = new EventEmitter();
        this.onChunkPreload = new EventEmitter();
        this._differ = null;
        this._lastTouchX = 0;
        this._lastTouchY = 0;
        this._embeddedViews = [];
    }
    IgxForOfDirective.prototype.ngOnInit = function () {
        var _this = this;
        var totalWidth = 0;
        var vc = this.igxForScrollContainer ? this.igxForScrollContainer._viewContainer : this._viewContainer;
        var dcFactory = this.resolver.resolveComponentFactory(DisplayContainerComponent);
        this.dc = this._viewContainer.createComponent(dcFactory, 0);
        if (this.igxForOf && this.igxForOf.length) {
            this.dc.instance.notVirtual = !(this.igxForContainerSize && this.state.chunkSize < this.igxForOf.length);
            if (this.igxForScrollOrientation === "horizontal") {
                totalWidth = this.initHCache(this.igxForOf);
                this.hScroll = this.getElement(vc, "igx-horizontal-virtual-helper");
                if (this.hScroll) {
                    this.state.startIndex = this.getHorizontalIndexAt(this.hScroll.scrollLeft, this.hCache, 0);
                }
            }
            this.state.chunkSize = this._calculateChunkSize();
            for (var i = 0; i < this.state.chunkSize && this.igxForOf[i] !== undefined; i++) {
                var input = this.igxForOf[i];
                var embeddedView = this.dc.instance._vcr.createEmbeddedView(this._template, { $implicit: input, index: this.igxForOf.indexOf(input) });
                this._embeddedViews.push(embeddedView);
            }
        }
        if (this.igxForScrollOrientation === "vertical") {
            var factory = this.resolver.resolveComponentFactory(VirtualHelperComponent);
            this.vh = this._viewContainer.createComponent(factory, 1);
            this.vh.instance.height = this.igxForOf ? this.igxForOf.length * parseInt(this.igxForItemSize, 10) : 0;
            this._zone.runOutsideAngular(function () {
                _this.vh.instance.elementRef.nativeElement.addEventListener("scroll", function (evt) { _this.onScroll(evt); });
                _this.dc.instance._viewContainer.element.nativeElement.addEventListener("wheel", function (evt) { _this.onWheel(evt); });
                _this.dc.instance._viewContainer.element.nativeElement.addEventListener("touchstart", function (evt) { _this.onTouchStart(evt); });
                _this.dc.instance._viewContainer.element.nativeElement.addEventListener("touchmove", function (evt) { _this.onTouchMove(evt); });
                _this.dc.instance._viewContainer.element.nativeElement.addEventListener("pointerdown", function (evt) { _this.onPointerDown(evt); });
                _this.dc.instance._viewContainer.element.nativeElement.addEventListener("pointerup", function (evt) { _this.onPointerUp(evt); });
                _this.dc.instance._viewContainer.element.nativeElement.addEventListener("MSGestureStart", function (evt) { _this.onMSGestureStart(evt); });
                _this.dc.instance._viewContainer.element.nativeElement.addEventListener("MSGestureChange", function (evt) { _this.onMSGestureChange(evt); });
            });
        }
        if (this.igxForScrollOrientation === "horizontal") {
            this.dc.instance._viewContainer.element.nativeElement.style.height = "100%";
            this.func = function (evt) { _this.onHScroll(evt); };
            if (!this.hScroll) {
                var hvFactory = this.resolver.resolveComponentFactory(HVirtualHelperComponent);
                this.hvh = vc.createComponent(hvFactory);
                this.hvh.instance.width = totalWidth;
                this.hScroll = this.hvh.instance.elementRef.nativeElement;
                this._zone.runOutsideAngular(function () {
                    _this.hvh.instance.elementRef.nativeElement.addEventListener("scroll", _this.func);
                });
            }
            else {
                this._zone.runOutsideAngular(function () {
                    _this.hScroll.addEventListener("scroll", _this.func);
                });
            }
        }
    };
    IgxForOfDirective.prototype.ngOnDestroy = function () {
        if (this.hScroll) {
            this.hScroll.removeEventListener("scroll", this.func);
        }
    };
    IgxForOfDirective.prototype.ngOnChanges = function (changes) {
        var forOf = "igxForOf";
        if (forOf in changes) {
            var value = changes[forOf].currentValue;
            if (!this._differ && value) {
                try {
                    this._differ = this._differs.find(value).create(this.ngForTrackBy);
                }
                catch (e) {
                    throw new Error("Cannot find a differ supporting object \"" + value + "\" of type \"" + getTypeNameForDebugging(value) + "\".\n                     NgFor only supports binding to Iterables such as Arrays.");
                }
            }
        }
        var containerSize = "igxForContainerSize";
        if (containerSize in changes && !changes[containerSize].firstChange) {
            this._recalcOnContainerChange(changes);
        }
    };
    IgxForOfDirective.prototype.ngDoCheck = function () {
        if (this._differ) {
            var changes = this._differ.diff(this.igxForOf);
            if (changes) {
                this._applyChanges(changes);
            }
        }
    };
    IgxForOfDirective.prototype.scrollTo = function (index) {
        if (index < 0 || index > this.igxForOf.length) {
            return;
        }
        this.state.startIndex = index;
        if (this.igxForScrollOrientation === "horizontal") {
            var sumWidths = this.igxForOf.reduce(function (acc, currentVal, currentIndex) {
                return (currentIndex <= index) ? acc + parseFloat(currentVal.width) : acc;
            }, 0);
            this.hScroll.scrollLeft = sumWidths;
        }
        else {
            this.vh.instance.elementRef.nativeElement.scrollTop = parseInt(this.igxForItemSize, 10) * index;
        }
    };
    IgxForOfDirective.prototype.scrollNext = function () {
        this.scrollTo(this.state.startIndex + 1);
    };
    IgxForOfDirective.prototype.scrollPrev = function () {
        this.scrollTo(this.state.startIndex - 1);
    };
    IgxForOfDirective.prototype.onScroll = function (event) {
        /* in certain situations this may be called when no scrollbar is visible */
        /* in certain situations this may be called when no scrollbar is visible */
        if (!parseInt(this.vh.instance.elementRef.nativeElement.style.height, 10)) {
            return;
        }
        var scrollTop = event.target.scrollTop;
        var vcHeight = event.target.children[0].scrollHeight;
        var ratio = vcHeight !== 0 ? scrollTop / vcHeight : 0;
        var embeddedViewCopy = Object.assign([], this._embeddedViews);
        var count = this.totalItemCount || this.igxForOf.length;
        var currIndex = Math.round(ratio * count);
        var endingIndex = this.state.chunkSize + currIndex;
        if (this.state.startIndex !== currIndex) {
            this.state.startIndex = currIndex;
            this.onChunkPreload.emit(this.state);
        }
        if (this.igxForRemote) {
            return;
        }
        if (endingIndex === this.igxForOf.length) {
            // scrolled to bottom
            var h = this.dc.instance._viewContainer.element.nativeElement.clientHeight;
            var totalDiff = h - this.state.chunkSize * parseInt(this.igxForItemSize, 10);
            if (totalDiff < 0) {
                this.dc.instance._viewContainer.element.nativeElement.firstElementChild.style.marginTop = totalDiff + "px";
            }
        }
        else {
            this.dc.instance._viewContainer.element.nativeElement.firstElementChild.style.marginTop = "";
        }
        for (var i = this.state.startIndex; i < endingIndex && this.igxForOf[i] !== undefined; i++) {
            var input = this.igxForOf[i];
            var embView = embeddedViewCopy.shift();
            var cntx = embView.context;
            cntx.$implicit = input;
            cntx.index = this.igxForOf.indexOf(input);
        }
        this.dc.changeDetectorRef.detectChanges();
        this.onChunkLoad.emit(this.state);
    };
    IgxForOfDirective.prototype.onHScroll = function (event) {
        /* in certain situations this may be called when no scrollbar is visible */
        /* in certain situations this may be called when no scrollbar is visible */
        if (!parseInt(this.hScroll.children[0].style.width, 10)) {
            return;
        }
        var scrollLeft = event.target.scrollLeft;
        this.state.startIndex = this.getHorizontalIndexAt(scrollLeft, this.hCache, 0);
        this.onChunkPreload.emit(this.state);
        /*recalculate and apply page size.*/
        this.applyChunkSizeChange();
        var embeddedViewCopy = Object.assign([], this._embeddedViews);
        var endingIndex = this.state.chunkSize + this.state.startIndex;
        for (var i = this.state.startIndex; i < endingIndex && this.igxForOf[i] !== undefined; i++) {
            var input = this.igxForOf[i];
            var embView = embeddedViewCopy.shift();
            var cntx = embView.context;
            cntx.$implicit = input;
            cntx.index = this.igxForOf.indexOf(input);
        }
        this.dc.changeDetectorRef.detectChanges();
        this.onChunkLoad.emit();
    };
    IgxForOfDirective.prototype.onWheel = function (event) {
        /** runs only on the vertical directive */
        var scrollStepX = 10;
        var scrollStepY = /Edge/.test(navigator.userAgent) ? 25 : 100;
        this.vh.instance.elementRef.nativeElement.scrollTop += Math.sign(event.deltaY) * scrollStepY;
        var hScroll = this.getElement(this._viewContainer, "igx-horizontal-virtual-helper");
        if (hScroll) {
            hScroll.scrollLeft += Math.sign(event.deltaX) * scrollStepX;
        }
        var curScrollTop = this.vh.instance.elementRef.nativeElement.scrollTop;
        var maxScrollTop = this.vh.instance.height - this.vh.instance.elementRef.nativeElement.offsetHeight;
        if (0 < curScrollTop && curScrollTop < maxScrollTop) {
            event.preventDefault();
        }
    };
    IgxForOfDirective.prototype.onTouchStart = function (event) {
        /** runs only on the vertical directive */
        /** runs only on the vertical directive */
        if (typeof MSGesture === "function") {
            return false;
        }
        this._lastTouchX = event.changedTouches[0].screenX;
        this._lastTouchY = event.changedTouches[0].screenY;
    };
    IgxForOfDirective.prototype.onTouchMove = function (event) {
        /** runs only on the vertical directive */
        /** runs only on the vertical directive */
        if (typeof MSGesture === "function") {
            return false;
        }
        var maxScrollTop = this.vh.instance.elementRef.nativeElement.children[0].offsetHeight -
            this.dc.instance._viewContainer.element.nativeElement.offsetHeight;
        var hScroll = this.getElement(this._viewContainer, "igx-horizontal-virtual-helper");
        var movedX = this._lastTouchX - event.changedTouches[0].screenX;
        var movedY = this._lastTouchY - event.changedTouches[0].screenY;
        if (hScroll) {
            hScroll.scrollLeft += movedX;
        }
        this.vh.instance.elementRef.nativeElement.scrollTop += movedY;
        if (this.vh.instance.elementRef.nativeElement.scrollTop !== 0 &&
            this.vh.instance.elementRef.nativeElement.scrollTop !== maxScrollTop) {
            event.preventDefault();
        }
        this._lastTouchX = event.changedTouches[0].screenX;
        this._lastTouchY = event.changedTouches[0].screenY;
    };
    IgxForOfDirective.prototype.onPointerDown = function (event) {
        /** runs only on the vertical directive */
        /** runs only on the vertical directive */
        if (!event || (event.pointerType !== 2 && event.pointerType !== "touch") ||
            typeof MSGesture !== "function") {
            return true;
        }
        if (!this._gestureObject) {
            this._gestureObject = new MSGesture();
            this._gestureObject.target = this.dc.instance._viewContainer.element.nativeElement;
        }
        event.target.setPointerCapture(this._pointerCapture = event.pointerId);
        this._gestureObject.addPointer(this._pointerCapture);
    };
    IgxForOfDirective.prototype.onPointerUp = function (event) {
        /** runs only on the vertical directive */
        /** runs only on the vertical directive */
        if (!this._pointerCapture) {
            return true;
        }
        event.target.releasePointerCapture(this._pointerCapture);
        delete this._pointerCapture;
    };
    IgxForOfDirective.prototype.onMSGestureStart = function (event) {
        /** runs only on the vertical directive */
        this._lastTouchX = event.screenX;
        this._lastTouchY = event.screenY;
        return false;
    };
    IgxForOfDirective.prototype.onMSGestureChange = function (event) {
        /** runs only on the vertical directive */
        var movedX = this._lastTouchX - event.screenX;
        var movedY = this._lastTouchY - event.screenY;
        var hScroll = this.getElement(this._viewContainer, "igx-horizontal-virtual-helper");
        if (hScroll) {
            hScroll.scrollLeft += movedX;
        }
        this.vh.instance.elementRef.nativeElement.scrollTop += movedY;
        this._lastTouchX = event.screenX;
        this._lastTouchY = event.screenY;
        return false;
    };
    Object.defineProperty(IgxForOfDirective.prototype, "ngForTrackBy", {
        get: function () { return this._trackByFn; },
        enumerable: true,
        configurable: true
    });
    IgxForOfDirective.prototype._applyChanges = function (changes) {
        this.applyChunkSizeChange();
        this._recalcScrollBarSize();
        if (this.igxForOf && this.igxForOf.length && this.dc) {
            var embeddedViewCopy = Object.assign([], this._embeddedViews);
            var startIndex = this.state.startIndex;
            var endIndex = this.state.chunkSize + this.state.startIndex;
            if (this.igxForRemote) {
                startIndex = 0;
                endIndex = this.igxForOf.length;
            }
            for (var i = startIndex; i < endIndex && this.igxForOf[i] !== undefined; i++) {
                var input = this.igxForOf[i];
                var embView = embeddedViewCopy.shift();
                var cntx = embView.context;
                cntx.$implicit = input;
                cntx.index = this.igxForOf.indexOf(input);
            }
            this.onChunkLoad.emit();
            this.dc.changeDetectorRef.detectChanges();
        }
    };
    IgxForOfDirective.prototype._calculateChunkSize = function () {
        var chunkSize = 0;
        if (this.igxForContainerSize !== null && this.igxForContainerSize !== undefined) {
            if (this.igxForScrollOrientation === "horizontal") {
                var vc = this.igxForScrollContainer ?
                    this.igxForScrollContainer._viewContainer :
                    this._viewContainer;
                var hScroll = this.getElement(vc, "igx-horizontal-virtual-helper");
                var left = hScroll && hScroll.scrollLeft !== 0 ?
                    hScroll.scrollLeft + parseInt(this.igxForContainerSize, 10) :
                    parseInt(this.igxForContainerSize, 10);
                if (!this.hCache) {
                    this.initHCache(this.igxForOf);
                }
                var endIndex = this.getHorizontalIndexAt(left, this.hCache, 0) + 1;
                if (endIndex >= this.igxForOf.length) {
                    endIndex = this.igxForOf.length;
                    /*At right edge. Check if last elem fits.*/
                    var diff = this.hCache[endIndex] - this.hCache[this.state.startIndex];
                    if (diff > parseInt(this.igxForContainerSize, 10)) {
                        /*If last col does not fit we should remove some of the prev cols to fit the last col.*/
                        /*If last col does not fit we should remove some of the prev cols to fit the last col.*/
                        while (diff > parseInt(this.igxForContainerSize, 10)) {
                            diff -= parseInt(this.igxForOf[this.state.startIndex].width, 10);
                            if (this.state.startIndex + 1 === endIndex) {
                                /*large column that exceeds the size of the container...*/
                                /*large column that exceeds the size of the container...*/
                                break;
                            }
                            this.state.startIndex++;
                        }
                    }
                }
                chunkSize = endIndex - this.state.startIndex;
            }
            else {
                chunkSize = Math.ceil(parseInt(this.igxForContainerSize, 10) /
                    parseInt(this.igxForItemSize, 10));
                if (chunkSize > this.igxForOf.length) {
                    chunkSize = this.igxForOf.length;
                }
            }
        }
        else {
            chunkSize = this.igxForOf.length;
        }
        return chunkSize;
    };
    IgxForOfDirective.prototype.getElement = function (viewref, nodeName) {
        var elem = viewref.element.nativeElement.parentNode.getElementsByTagName(nodeName);
        return elem.length > 0 ? elem[0] : null;
    };
    IgxForOfDirective.prototype.initHCache = function (cols) {
        var totalWidth = 0;
        var i = 0;
        this.hCache = [];
        this.hCache.push(0);
        for (i; i < cols.length; i++) {
            totalWidth += parseInt(cols[i].width, 10) || 0;
            this.hCache.push(totalWidth);
        }
        return totalWidth;
    };
    IgxForOfDirective.prototype.getHorizontalIndexAt = function (left, set, index) {
        var midIdx;
        var midLeft;
        if (set.length === 1) {
            return index;
        }
        midIdx = Math.floor(set.length / 2);
        midLeft = set[midIdx];
        return this.getHorizontalIndexAt(left, midLeft >= left ? set.slice(0, midIdx) : set.slice(midIdx), midLeft >= left ? index : index + midIdx);
    };
    IgxForOfDirective.prototype._recalcScrollBarSize = function () {
        this.dc.instance.notVirtual = !(this.igxForContainerSize && this.dc && this.state.chunkSize < this.igxForOf.length);
        if (this.igxForScrollOrientation === "horizontal") {
            var totalWidth = this.igxForContainerSize ? this.initHCache(this.igxForOf) : 0;
            this.hScroll.children[0].style.width = totalWidth + "px";
        }
        if (this.igxForScrollOrientation === "vertical") {
            var count = this.totalItemCount || this.igxForOf.length;
            this.vh.instance.elementRef.nativeElement.style.height = parseInt(this.igxForContainerSize, 10) + "px";
            this.vh.instance.height = count * parseInt(this.igxForItemSize, 10);
        }
    };
    IgxForOfDirective.prototype._recalcOnContainerChange = function (changes) {
        this.applyChunkSizeChange();
        this._recalcScrollBarSize();
    };
    IgxForOfDirective.prototype.applyChunkSizeChange = function () {
        var chunkSize = this._calculateChunkSize();
        if (chunkSize > this.state.chunkSize) {
            var diff = chunkSize - this.state.chunkSize;
            for (var i = 0; i < diff; i++) {
                var input = this.igxForOf[this.state.startIndex + this.state.chunkSize + i];
                var embeddedView = this.dc.instance._vcr.createEmbeddedView(this._template, { $implicit: input, index: this.igxForOf.indexOf(input) });
                this._embeddedViews.push(embeddedView);
            }
        }
        else if (chunkSize < this.state.chunkSize) {
            var diff = this.state.chunkSize - chunkSize;
            for (var i = 0; i < diff; i++) {
                var ind = this.state.chunkSize - i - 1;
                var embeddedView = this._embeddedViews[ind];
                embeddedView.destroy();
                this._embeddedViews.splice(ind, 1);
            }
        }
        this.state.chunkSize = chunkSize;
    };
    IgxForOfDirective.decorators = [
        { type: Directive, args: [{ selector: "[igxFor][igxForOf]" },] },
    ];
    /** @nocollapse */
    IgxForOfDirective.ctorParameters = function () { return [
        { type: ViewContainerRef, },
        { type: TemplateRef, },
        { type: IterableDiffers, },
        { type: ComponentFactoryResolver, },
        { type: ChangeDetectorRef, },
        { type: NgZone, },
    ]; };
    IgxForOfDirective.propDecorators = {
        "igxForOf": [{ type: Input },],
        "igxForScrollOrientation": [{ type: Input },],
        "igxForScrollContainer": [{ type: Input },],
        "igxForContainerSize": [{ type: Input },],
        "igxForItemSize": [{ type: Input },],
        "igxForRemote": [{ type: Input },],
        "onChunkLoad": [{ type: Output },],
        "onChunkPreload": [{ type: Output },],
        "displayContiner": [{ type: ViewChild, args: [DisplayContainerComponent,] },],
        "virtualHelper": [{ type: ViewChild, args: [VirtualHelperComponent,] },],
    };
    return IgxForOfDirective;
}());
export { IgxForOfDirective };
export function getTypeNameForDebugging(type) {
    var name = "name";
    return type[name] || typeof type;
}
var IgxForOfModule = /** @class */ (function () {
    function IgxForOfModule() {
    }
    IgxForOfModule.decorators = [
        { type: NgModule, args: [{
                    declarations: [IgxForOfDirective, DisplayContainerComponent, VirtualHelperComponent, HVirtualHelperComponent],
                    entryComponents: [DisplayContainerComponent, VirtualHelperComponent, HVirtualHelperComponent],
                    exports: [IgxForOfDirective],
                    imports: [CommonModule]
                },] },
    ];
    /** @nocollapse */
    IgxForOfModule.ctorParameters = function () { return []; };
    return IgxForOfModule;
}());
export { IgxForOfModule };
//# sourceMappingURL=for_of.directive.js.map