import { AfterContentInit, AfterViewInit, ChangeDetectorRef, ComponentFactoryResolver, ElementRef, EventEmitter, NgZone, OnDestroy, OnInit, QueryList, TemplateRef, ViewContainerRef } from "@angular/core";
import { Subject } from "rxjs/Subject";
import { DataType } from "../data-operations/data-util";
import { FilteringLogic, IFilteringExpression } from "../data-operations/filtering-expression.interface";
import { ISortingExpression, SortingDirection } from "../data-operations/sorting-expression.interface";
import { IgxForOfDirective } from "../directives/for-of/for_of.directive";
import { IgxGridAPIService } from "./api.service";
import { IgxGridCellComponent } from "./cell.component";
import { IgxColumnComponent } from "./column.component";
import { ISummaryExpression } from "./grid-summary";
import { IgxGridRowComponent } from "./row.component";
export declare class IgxGridComponent implements OnInit, OnDestroy, AfterContentInit, AfterViewInit {
    private gridAPI;
    private elementRef;
    private zone;
    private document;
    cdr: ChangeDetectorRef;
    private resolver;
    private viewRef;
    data: any[];
    autoGenerate: boolean;
    id: string;
    filteringLogic: FilteringLogic;
    filteringExpressions: any[];
    paging: boolean;
    page: number;
    perPage: number;
    paginationTemplate: TemplateRef<any>;
    height: any;
    width: any;
    readonly headerWidth: number;
    evenRowCSS: string;
    oddRowCSS: string;
    rowHeight: number;
    onSelection: EventEmitter<any>;
    onColumnPinning: EventEmitter<any>;
    onEditDone: EventEmitter<any>;
    onColumnInit: EventEmitter<any>;
    onSortingDone: EventEmitter<any>;
    onFilteringDone: EventEmitter<any>;
    onPagingDone: EventEmitter<any>;
    onRowAdded: EventEmitter<any>;
    onRowDeleted: EventEmitter<any>;
    columnList: QueryList<IgxColumnComponent>;
    rowList: QueryList<IgxGridRowComponent>;
    parentVirtDir: IgxForOfDirective<any>;
    verticalScrollContainer: IgxForOfDirective<any>;
    scr: ElementRef;
    paginator: ElementRef;
    headerContainer: IgxForOfDirective<any>;
    theadRow: ElementRef;
    tfoot: ElementRef;
    tabindex: number;
    hostClass: string;
    hostRole: string;
    readonly pipeTrigger: number;
    sortingExpressions: any[];
    pagingState: any;
    calcWidth: number;
    calcHeight: number;
    cellInEditMode: IgxGridCellComponent;
    eventBus: Subject<boolean>;
    protected destroy$: Subject<boolean>;
    protected _perPage: number;
    protected _page: number;
    protected _paging: boolean;
    protected _pipeTrigger: number;
    protected _columns: IgxColumnComponent[];
    protected _pinnedColumns: IgxColumnComponent[];
    protected _unpinnedColumns: IgxColumnComponent[];
    protected _filteringLogic: FilteringLogic;
    protected _filteringExpressions: any[];
    protected _sortingExpressions: any[];
    private resizeHandler;
    constructor(gridAPI: IgxGridAPIService, elementRef: ElementRef, zone: NgZone, document: any, cdr: ChangeDetectorRef, resolver: ComponentFactoryResolver, viewRef: ViewContainerRef);
    ngOnInit(): void;
    ngAfterContentInit(): void;
    ngAfterViewInit(): void;
    ngOnDestroy(): void;
    readonly nativeElement: any;
    readonly pinnedWidth: number;
    readonly unpinnedWidth: number;
    readonly columns: IgxColumnComponent[];
    readonly pinnedColumns: IgxColumnComponent[];
    readonly unpinnedColumns: IgxColumnComponent[];
    getColumnByName(name: string): IgxColumnComponent;
    getRowByIndex(index: number): IgxGridRowComponent;
    readonly visibleColumns: IgxColumnComponent[];
    getCellByColumn(rowIndex: number, columnField: string): IgxGridCellComponent;
    readonly totalPages: number;
    readonly totalRecords: number;
    readonly isFirstPage: boolean;
    readonly isLastPage: boolean;
    nextPage(): void;
    previousPage(): void;
    paginate(val: number): void;
    markForCheck(): void;
    addRow(data: any): void;
    deleteRow(rowIndex: number): void;
    updateCell(value: any, rowIndex: number, column: string): void;
    updateRow(value: any, rowIndex: number): void;
    sort(...rest: any[]): void;
    filter(...rest: any[]): void;
    filterGlobal(value: any, condition?: any, ignoreCase?: any): void;
    enableSummaries(...rest: any[]): void;
    disableSummaries(...rest: any[]): void;
    clearFilter(name?: string): void;
    clearSort(name?: string): void;
    clearSummaryCache(): void;
    pinColumn(columnName: string): boolean;
    unpinColumn(columnName: string): void;
    readonly hasSortableColumns: boolean;
    readonly hasEditableColumns: boolean;
    readonly hasFilterableColumns: boolean;
    readonly hasSummarizedColumns: boolean;
    readonly selectedCells: IgxGridCellComponent[] | any[];
    protected calculateGridSizes(): void;
    /**
     * Gets calculated width of the start pinned area
     * @param takeHidden If we should take into account the hidden columns in the pinned area
     */
    protected getPinnedWidth(takeHidden?: boolean): number;
    /**
     * Gets calculated width of the unpinned area
     * @param takeHidden If we should take into account the hidden columns in the pinned area
     */
    protected getUnpinnedWidth(takeHidden?: boolean): number;
    protected _sort(name: string, direction?: SortingDirection, ignoreCase?: boolean): void;
    protected _sortMultiple(expressions: ISortingExpression[]): void;
    protected _filter(name: string, value: any, condition?: any, ignoreCase?: any): void;
    protected _filterMultiple(expressions: IFilteringExpression[]): void;
    protected _summaries(fieldName: string, hasSummary: boolean, summaryOperand?: any): void;
    protected _multipleSummaries(expressions: ISummaryExpression[], hasSummary: boolean): void;
    protected _disableMultipleSummaries(expressions: string[], hasSummary: boolean): void;
    protected resolveDataTypes(rec: any): DataType;
    protected autogenerateColumns(): void;
    protected setEventBusSubscription(): void;
    protected setVerticalScrollSubscription(): void;
}
