import { DOCUMENT } from "@angular/common";
import { ChangeDetectionStrategy, ChangeDetectorRef, Component, ComponentFactoryResolver, ContentChildren, ElementRef, EventEmitter, HostBinding, Inject, Input, NgZone, Output, QueryList, TemplateRef, ViewChild, ViewChildren, ViewContainerRef } from "@angular/core";
import { of } from "rxjs/observable/of";
import { debounceTime, delay, merge, repeat, take, takeUntil } from "rxjs/operators";
import { Subject } from "rxjs/Subject";
import { cloneArray } from "../core/utils";
import { DataType } from "../data-operations/data-util";
import { FilteringLogic } from "../data-operations/filtering-expression.interface";
import { SortingDirection } from "../data-operations/sorting-expression.interface";
import { IgxForOfDirective } from "../directives/for-of/for_of.directive";
import { IgxGridAPIService } from "./api.service";
import { IgxColumnComponent } from "./column.component";
import { IgxGridRowComponent } from "./row.component";
var NEXT_ID = 0;
var DEBOUNCE_TIME = 16;
var IgxGridComponent = /** @class */ (function () {
    function IgxGridComponent(gridAPI, elementRef, zone, document, cdr, resolver, viewRef) {
        var _this = this;
        this.gridAPI = gridAPI;
        this.elementRef = elementRef;
        this.zone = zone;
        this.document = document;
        this.cdr = cdr;
        this.resolver = resolver;
        this.viewRef = viewRef;
        this.data = [];
        this.autoGenerate = false;
        this.id = "igx-grid-" + NEXT_ID++;
        this.filteringLogic = FilteringLogic.And;
        this.evenRowCSS = "";
        this.oddRowCSS = "";
        this.rowHeight = 50;
        this.onSelection = new EventEmitter();
        this.onColumnPinning = new EventEmitter();
        this.onEditDone = new EventEmitter();
        this.onColumnInit = new EventEmitter();
        this.onSortingDone = new EventEmitter();
        this.onFilteringDone = new EventEmitter();
        this.onPagingDone = new EventEmitter();
        this.onRowAdded = new EventEmitter();
        this.onRowDeleted = new EventEmitter();
        this.tabindex = 0;
        this.hostClass = "igx-grid";
        this.hostRole = "grid";
        this.eventBus = new Subject();
        this.destroy$ = new Subject();
        this._perPage = 15;
        this._page = 0;
        this._paging = false;
        this._pipeTrigger = 0;
        this._columns = [];
        this._pinnedColumns = [];
        this._unpinnedColumns = [];
        this._filteringLogic = FilteringLogic.And;
        this._filteringExpressions = [];
        this._sortingExpressions = [];
        this.resizeHandler = function () {
            _this.calculateGridSizes();
            _this.zone.run(function () { return _this.markForCheck(); });
        };
    }
    Object.defineProperty(IgxGridComponent.prototype, "filteringExpressions", {
        get: function () {
            return this._filteringExpressions;
        },
        set: function (value) {
            this._filteringExpressions = cloneArray(value);
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "paging", {
        get: function () {
            return this._paging;
        },
        set: function (value) {
            this._paging = value;
            this._pipeTrigger++;
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "page", {
        get: function () {
            return this._page;
        },
        set: function (val) {
            if (val < 0) {
                return;
            }
            this.onPagingDone.emit({ previous: this._page, current: val });
            this._page = val;
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "perPage", {
        get: function () {
            return this._perPage;
        },
        set: function (val) {
            if (val < 0) {
                return;
            }
            this._perPage = val;
            this.page = 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "headerWidth", {
        get: function () {
            return parseInt(this.width, 10) - 17;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "pipeTrigger", {
        get: function () {
            return this._pipeTrigger;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "sortingExpressions", {
        get: function () {
            return this._sortingExpressions;
        },
        set: function (value) {
            this._sortingExpressions = cloneArray(value);
            this.cdr.markForCheck();
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.ngOnInit = function () {
        this.gridAPI.register(this);
        this.calcWidth = this.width && this.width.indexOf("%") === -1 ? parseInt(this.width, 10) : 0;
        this.calcHeight = 0;
    };
    IgxGridComponent.prototype.ngAfterContentInit = function () {
        var _this = this;
        if (this.autoGenerate) {
            this.autogenerateColumns();
        }
        this.columnList.forEach(function (col, idx) {
            col.index = idx;
            col.gridID = _this.id;
            _this.onColumnInit.emit(col);
        });
        this._columns = this.columnList.toArray();
        this._pinnedColumns = this._columns.filter(function (c) { return c.pinned; });
        this._unpinnedColumns = this._columns.filter(function (c) { return !c.pinned; });
    };
    IgxGridComponent.prototype.ngAfterViewInit = function () {
        var _this = this;
        this.zone.runOutsideAngular(function () {
            _this.document.defaultView.addEventListener("resize", _this.resizeHandler);
        });
        this.calculateGridSizes();
        this.setEventBusSubscription();
        this.setVerticalScrollSubscription();
        this.cdr.detectChanges();
    };
    IgxGridComponent.prototype.ngOnDestroy = function () {
        var _this = this;
        this.zone.runOutsideAngular(function () { return _this.document.defaultView.removeEventListener("resize", _this.resizeHandler); });
        this.destroy$.next(true);
        this.destroy$.complete();
    };
    Object.defineProperty(IgxGridComponent.prototype, "nativeElement", {
        get: function () {
            return this.elementRef.nativeElement;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "pinnedWidth", {
        get: function () {
            return this.getPinnedWidth();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "unpinnedWidth", {
        get: function () {
            return this.getUnpinnedWidth();
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "columns", {
        get: function () {
            return this._columns;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "pinnedColumns", {
        get: function () {
            return this._pinnedColumns.filter(function (col) { return !col.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "unpinnedColumns", {
        get: function () {
            return this._unpinnedColumns.filter(function (col) { return !col.hidden; }).sort(function (col1, col2) { return col1.index - col2.index; });
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.getColumnByName = function (name) {
        return this.columnList.find(function (col) { return col.field === name; });
    };
    IgxGridComponent.prototype.getRowByIndex = function (index) {
        return this.rowList.toArray()[index];
    };
    Object.defineProperty(IgxGridComponent.prototype, "visibleColumns", {
        get: function () {
            return this.columnList.filter(function (col) { return !col.hidden; });
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.getCellByColumn = function (rowIndex, columnField) {
        return this.gridAPI.get_cell_by_field(this.id, rowIndex, columnField);
    };
    Object.defineProperty(IgxGridComponent.prototype, "totalPages", {
        get: function () {
            if (this.pagingState) {
                return this.pagingState.metadata.countPages;
            }
            return -1;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "totalRecords", {
        get: function () {
            if (this.pagingState) {
                return this.pagingState.metadata.countRecords;
            }
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "isFirstPage", {
        get: function () {
            return this.page === 0;
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "isLastPage", {
        get: function () {
            return this.page + 1 >= this.totalPages;
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.nextPage = function () {
        if (!this.isLastPage) {
            this.page += 1;
        }
    };
    IgxGridComponent.prototype.previousPage = function () {
        if (!this.isFirstPage) {
            this.page -= 1;
        }
    };
    IgxGridComponent.prototype.paginate = function (val) {
        if (val < 0) {
            return;
        }
        this.page = val;
    };
    IgxGridComponent.prototype.markForCheck = function () {
        if (this.rowList) {
            this.rowList.forEach(function (row) { return row.cdr.markForCheck(); });
        }
        this.cdr.detectChanges();
    };
    IgxGridComponent.prototype.addRow = function (data) {
        this.data.push(data);
        this.onRowAdded.emit({ data: data });
        this._pipeTrigger++;
        this.cdr.markForCheck();
    };
    IgxGridComponent.prototype.deleteRow = function (rowIndex) {
        var row = this.gridAPI.get_row(this.id, rowIndex);
        if (row) {
            var index = this.data.indexOf(row.rowData);
            this.data.splice(index, 1);
            this.onRowDeleted.emit({ row: row });
            this._pipeTrigger++;
            this.cdr.markForCheck();
        }
    };
    IgxGridComponent.prototype.updateCell = function (value, rowIndex, column) {
        var cell = this.gridAPI.get_cell_by_field(this.id, rowIndex, column);
        if (cell) {
            cell.update(value);
            this._pipeTrigger++;
        }
    };
    IgxGridComponent.prototype.updateRow = function (value, rowIndex) {
        var row = this.gridAPI.get_row(this.id, rowIndex);
        if (row) {
            this.gridAPI.update_row(value, this.id, row);
            this._pipeTrigger++;
            this.cdr.markForCheck();
        }
    };
    IgxGridComponent.prototype.sort = function () {
        var rest = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            rest[_i] = arguments[_i];
        }
        if (rest.length === 1 && rest[0] instanceof Array) {
            this._sortMultiple(rest[0]);
        }
        else {
            this._sort(rest[0], rest[1], rest[2]);
        }
    };
    IgxGridComponent.prototype.filter = function () {
        var rest = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            rest[_i] = arguments[_i];
        }
        if (rest.length === 1 && rest[0] instanceof Array) {
            this._filterMultiple(rest[0]);
        }
        else {
            this._filter(rest[0], rest[1], rest[2], rest[3]);
        }
    };
    IgxGridComponent.prototype.filterGlobal = function (value, condition, ignoreCase) {
        this.gridAPI.filter_global(this.id, value, condition, ignoreCase);
    };
    IgxGridComponent.prototype.enableSummaries = function () {
        var rest = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            rest[_i] = arguments[_i];
        }
        if (rest.length === 1 && Array.isArray(rest[0])) {
            this._multipleSummaries(rest[0], true);
        }
        else {
            this._summaries(rest[0], true, rest[1]);
        }
        this.markForCheck();
        this.calculateGridSizes();
    };
    IgxGridComponent.prototype.disableSummaries = function () {
        var rest = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            rest[_i] = arguments[_i];
        }
        if (rest.length === 1 && Array.isArray(rest[0])) {
            this._disableMultipleSummaries(rest[0], false);
        }
        else {
            this._summaries(rest[0], false);
        }
        this.markForCheck();
        this.calculateGridSizes();
    };
    IgxGridComponent.prototype.clearFilter = function (name) {
        if (!name) {
            this.filteringExpressions = [];
            return;
        }
        if (!this.gridAPI.get_column_by_name(this.id, name)) {
            return;
        }
        this.gridAPI.clear_filter(this.id, name);
    };
    IgxGridComponent.prototype.clearSort = function (name) {
        if (!name) {
            this.sortingExpressions = [];
            return;
        }
        if (!this.gridAPI.get_column_by_name(this.id, name)) {
            return;
        }
        this.gridAPI.clear_sort(this.id, name);
    };
    IgxGridComponent.prototype.clearSummaryCache = function () {
        this.gridAPI.remove_summary(this.id);
    };
    IgxGridComponent.prototype.pinColumn = function (columnName) {
        var col = this.getColumnByName(columnName);
        /**
                 * If the column that we want to pin is bigger or equal than the unpinned area we should not pin it.
                 * It should be also unpinned before pinning, since changing left/right pin area doesn't affect unpinned area.
                 */
        /**
         * If the column that we want to pin is bigger or equal than the unpinned area we should not pin it.
         * It should be also unpinned before pinning, since changing left/right pin area doesn't affect unpinned area.
         */
        if (parseInt(col.width, 10) >= this.getUnpinnedWidth(true) && !col.pinned) {
            return false;
        }
        col.pinned = true;
        var index = this._pinnedColumns.length;
        var args = { column: col, insertAtIndex: index };
        this.onColumnPinning.emit(args);
        // update grid collections.
        // update grid collections.
        if (this._pinnedColumns.indexOf(col) === -1) {
            this._pinnedColumns.splice(args.insertAtIndex, 0, col);
            if (this._unpinnedColumns.indexOf(col) !== -1) {
                this._unpinnedColumns.splice(this._unpinnedColumns.indexOf(col), 1);
            }
        }
        this.markForCheck();
        return true;
    };
    IgxGridComponent.prototype.unpinColumn = function (columnName) {
        var col = this.getColumnByName(columnName);
        col.pinned = false;
        this._unpinnedColumns.splice(col.index, 0, col);
        if (this._pinnedColumns.indexOf(col) !== -1) {
            this._pinnedColumns.splice(this._pinnedColumns.indexOf(col), 1);
        }
        this.markForCheck();
    };
    Object.defineProperty(IgxGridComponent.prototype, "hasSortableColumns", {
        get: function () {
            return this.columnList.some(function (col) { return col.sortable; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasEditableColumns", {
        get: function () {
            return this.columnList.some(function (col) { return col.editable; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasFilterableColumns", {
        get: function () {
            return this.columnList.some(function (col) { return col.filterable; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "hasSummarizedColumns", {
        get: function () {
            return this.columnList.some(function (col) { return col.hasSummary; });
        },
        enumerable: true,
        configurable: true
    });
    Object.defineProperty(IgxGridComponent.prototype, "selectedCells", {
        get: function () {
            if (this.rowList) {
                return this.rowList.map(function (row) { return row.cells.filter(function (cell) { return cell.selected; }); })
                    .reduce(function (a, b) { return a.concat(b); }, []);
            }
            return [];
        },
        enumerable: true,
        configurable: true
    });
    IgxGridComponent.prototype.calculateGridSizes = function () {
        var computed = this.document.defaultView.getComputedStyle(this.nativeElement);
        if (!this.width) {
            /*no width specified.*/
            this.calcWidth = null;
        }
        else if (this.width && this.width.indexOf("%") !== -1) {
            /* width in %*/
            this.calcWidth = parseInt(computed.getPropertyValue("width"), 10);
        }
        if (!this.height) {
            /*no height specified.*/
            this.calcHeight = null;
        }
        else if (this.height && this.height.indexOf("%") !== -1) {
            /*height in %*/
            var pagingHeight = 0;
            if (this.paging) {
                pagingHeight = this.paginator.nativeElement.firstElementChild ?
                    this.paginator.nativeElement.clientHeight : 0;
            }
            var footerHeight = this.tfoot.nativeElement.firstElementChild ?
                this.tfoot.nativeElement.clientHeight : 0;
            this.calcHeight = parseInt(computed.getPropertyValue("height"), 10) -
                this.theadRow.nativeElement.clientHeight -
                footerHeight - pagingHeight -
                this.scr.nativeElement.clientHeight;
        }
        else {
            var pagingHeight = 0;
            if (this.paging) {
                pagingHeight = this.paginator.nativeElement.firstElementChild ?
                    this.paginator.nativeElement.clientHeight : 0;
            }
            var footerHeight = this.tfoot.nativeElement.firstElementChild ?
                this.tfoot.nativeElement.clientHeight : 0;
            this.calcHeight = parseInt(this.height, 10) -
                this.theadRow.nativeElement.getBoundingClientRect().height -
                footerHeight - pagingHeight -
                this.scr.nativeElement.clientHeight;
        }
        this.cdr.detectChanges();
    };
    /**
     * Gets calculated width of the start pinned area
     * @param takeHidden If we should take into account the hidden columns in the pinned area
     */
    /**
         * Gets calculated width of the start pinned area
         * @param takeHidden If we should take into account the hidden columns in the pinned area
         */
    IgxGridComponent.prototype.getPinnedWidth = /**
         * Gets calculated width of the start pinned area
         * @param takeHidden If we should take into account the hidden columns in the pinned area
         */
    function (takeHidden) {
        if (takeHidden === void 0) { takeHidden = false; }
        var fc = takeHidden ? this._pinnedColumns : this.pinnedColumns;
        var sum = 0;
        for (var _i = 0, fc_1 = fc; _i < fc_1.length; _i++) {
            var col = fc_1[_i];
            sum += parseInt(col.width, 10);
        }
        return sum;
    };
    /**
     * Gets calculated width of the unpinned area
     * @param takeHidden If we should take into account the hidden columns in the pinned area
     */
    /**
         * Gets calculated width of the unpinned area
         * @param takeHidden If we should take into account the hidden columns in the pinned area
         */
    IgxGridComponent.prototype.getUnpinnedWidth = /**
         * Gets calculated width of the unpinned area
         * @param takeHidden If we should take into account the hidden columns in the pinned area
         */
    function (takeHidden) {
        if (takeHidden === void 0) { takeHidden = false; }
        var width = this.width && this.width.indexOf("%") !== -1 ?
            this.calcWidth :
            parseInt(this.width, 10);
        return width - this.getPinnedWidth(takeHidden);
    };
    IgxGridComponent.prototype._sort = function (name, direction, ignoreCase) {
        if (direction === void 0) { direction = SortingDirection.Asc; }
        if (ignoreCase === void 0) { ignoreCase = true; }
        this.gridAPI.sort(this.id, name, direction, ignoreCase);
    };
    IgxGridComponent.prototype._sortMultiple = function (expressions) {
        this.gridAPI.sort_multiple(this.id, expressions);
    };
    IgxGridComponent.prototype._filter = function (name, value, condition, ignoreCase) {
        var col = this.gridAPI.get_column_by_name(this.id, name);
        if (col) {
            this.gridAPI
                .filter(this.id, name, value, condition || col.filteringCondition, ignoreCase || col.filteringIgnoreCase);
        }
        else {
            this.gridAPI.filter(this.id, name, value, condition, ignoreCase);
        }
    };
    IgxGridComponent.prototype._filterMultiple = function (expressions) {
        this.gridAPI.filter_multiple(this.id, expressions);
    };
    IgxGridComponent.prototype._summaries = function (fieldName, hasSummary, summaryOperand) {
        var column = this.gridAPI.get_column_by_name(this.id, fieldName);
        column.hasSummary = hasSummary;
        if (summaryOperand) {
            column.summaries = summaryOperand;
        }
    };
    IgxGridComponent.prototype._multipleSummaries = function (expressions, hasSummary) {
        var _this = this;
        expressions.forEach(function (element) {
            _this._summaries(element.fieldName, hasSummary, element.customSummary);
        });
    };
    IgxGridComponent.prototype._disableMultipleSummaries = function (expressions, hasSummary) {
        var _this = this;
        expressions.forEach(function (column) { _this._summaries(column, false); });
    };
    IgxGridComponent.prototype.resolveDataTypes = function (rec) {
        if (typeof rec === "number") {
            return DataType.Number;
        }
        else if (typeof rec === "boolean") {
            return DataType.Boolean;
        }
        else if (typeof rec === "object" && rec instanceof Date) {
            return DataType.Date;
        }
        return DataType.String;
    };
    IgxGridComponent.prototype.autogenerateColumns = function () {
        var _this = this;
        var factory = this.resolver.resolveComponentFactory(IgxColumnComponent);
        var fields = Object.keys(this.data[0]);
        var columns = [];
        fields.forEach(function (field) {
            var ref = _this.viewRef.createComponent(factory);
            ref.instance.field = field;
            ref.instance.dataType = _this.resolveDataTypes(_this.data[0][field]);
            ref.changeDetectorRef.detectChanges();
            columns.push(ref.instance);
        });
        this.columnList.reset(columns);
    };
    IgxGridComponent.prototype.setEventBusSubscription = function () {
        var _this = this;
        this.eventBus.pipe(debounceTime(DEBOUNCE_TIME), takeUntil(this.destroy$)).subscribe(function () { return _this.cdr.detectChanges(); });
    };
    IgxGridComponent.prototype.setVerticalScrollSubscription = function () {
        var _this = this;
        /*
                    Until the grid component is destroyed,
                    Take the first event and unsubscribe
                    then merge with an empty observable after DEBOUNCE_TIME,
                    re-subscribe and repeat the process
                */
        this.parentVirtDir.onChunkLoad.pipe(takeUntil(this.destroy$), take(1), merge(of({})), delay(DEBOUNCE_TIME), repeat()).subscribe(function () {
            if (_this.cellInEditMode) {
                _this.cellInEditMode.inEditMode = false;
            }
        });
    };
    IgxGridComponent.decorators = [
        { type: Component, args: [{
                    changeDetection: ChangeDetectionStrategy.OnPush,
                    preserveWhitespaces: false,
                    selector: "igx-grid",
                    template: "<ng-template #defaultPager let-api>     <ng-container *ngIf=\"api.paging && api.totalPages > 0\">         <button [disabled]=\"api.isFirstPage\" (click)=\"api.paginate(0)\" igxButton=\"icon\" igxRipple igxRippleCentered=\"true\">             <igx-icon fontSet=\"material\" name=\"first_page\"></igx-icon>         </button>         <button [disabled]=\"api.isFirstPage\" (click)=\"api.previousPage()\" igxButton=\"icon\" igxRipple igxRippleCentered=\"true\">             <igx-icon fontSet=\"material\" name=\"chevron_left\"></igx-icon>         </button>         <span>{{ api.page + 1 }} of {{ api.totalPages }}</span>         <button [disabled]=\"api.isLastPage\" (click)=\"api.nextPage()\" igxRipple igxRippleCentered=\"true\" igxButton=\"icon\">             <igx-icon fontSet=\"material\" name=\"chevron_right\"></igx-icon>         </button>         <button [disabled]=\"api.isLastPage\" (click)=\"api.paginate(api.totalPages - 1)\" igxButton=\"icon\" igxRipple igxRippleCentered=\"true\">             <igx-icon fontSet=\"material\" name=\"last_page\"></igx-icon>         </button>         <select style=\"margin-left: 1rem;\" (change)=\"api.perPage = $event.target.value\">             <option [value]=\"val\" [selected]=\"api.perPage == val\" *ngFor=\"let val of [5, 10, 15, 25, 50, 100, 500]\">{{ val }}</option>         </select>     </ng-container> </ng-template>  <div class=\"igx-grid__thead\" role=\"rowgroup\" [style.width.px]='calcWidth' #theadRow>     <div class=\"igx-grid__tr\" role=\"row\" style=\"width: calc(100% - 18px)\">         <ng-container *ngIf=\"pinnedColumns.length > 0\">             <igx-grid-header [gridID]=\"id\" *ngFor=\"let col of pinnedColumns\" [column]=\"col\" [style.min-width.px]=\"col.width\"></igx-grid-header>         </ng-container>         <ng-template igxFor let-col [igxForOf]=\"unpinnedColumns\" [igxForScrollOrientation]=\"'horizontal'\" [igxForScrollContainer]=\"parentVirtDir\"             [igxForContainerSize]='unpinnedWidth' #headerContainer>             <igx-grid-header [gridID]=\"id\" [column]=\"col\" [style.min-width.px]=\"col.width\"></igx-grid-header>         </ng-template>     </div> </div>  <div class=\"igx-grid__tbody\" role=\"rowgroup\" [style.height.px]='calcHeight' [style.width.px]='calcWidth'>     <ng-template igxFor let-rowData [igxForOf]=\"data | gridFiltering:filteringExpressions:filteringLogic:id:pipeTrigger                             | gridSort:sortingExpressions:id:pipeTrigger                             | gridPaging:page:perPage:id:pipeTrigger\" let-rowIndex=\"index\" [igxForScrollOrientation]=\"'vertical'\"         [igxForContainerSize]='calcHeight' [igxForItemSize]=\"rowHeight\" #verticalScrollContainer>         <igx-grid-row [gridID]=\"id\" [index]=\"rowIndex\" [rowData]=\"rowData\">         </igx-grid-row>     </ng-template> </div>   <div class=\"igx-grid__tfoot\" role=\"rowgroup\" [style.width.px]='calcWidth' #tfoot>     <div *ngIf=\"hasSummarizedColumns\" class=\"igx-grid__tr\" role=\"row\">         <ng-container *ngIf=\"pinnedColumns.length > 0\">             <igx-grid-summary [gridID]=\"id\" *ngFor=\"let col of pinnedColumns\"  [column]=\"col\" [style.min-width.px]=\"col.width\"></igx-grid-summary>         </ng-container>         <ng-template igxFor let-col [igxForOf]=\"unpinnedColumns\" [igxForScrollOrientation]=\"'horizontal'\" [igxForScrollContainer]=\"parentVirtDir\" [igxForContainerSize]='unpinnedWidth' #summaryContainer>             <igx-grid-summary [gridID]=\"id\" [column]=\"col\" [style.min-width.px]=\"col.width\"></igx-grid-summary>         </ng-template>     </div> </div>  <div class=\"igx-grid__scroll\" [style.height]=\"'18px'\" #scr>     <div class=\"igx-grid__scroll-start\" [style.width.px]='pinnedWidth' [hidden]=\"pinnedWidth === 0\"></div>     <div class=\"igx-grid__scroll-main\" [style.width.px]='unpinnedWidth'>         <ng-template igxFor [igxForOf]='[]' #scrollContainer>         </ng-template>     </div> </div>  <div class=\"igx-paginator igx-grid-paginator\" *ngIf=\"paging && totalPages > 0\" #paginator>     <ng-container *ngTemplateOutlet=\"paginationTemplate ? paginationTemplate : defaultPager; context: { $implicit: this }\">     </ng-container> </div>"
                },] },
    ];
    /** @nocollapse */
    IgxGridComponent.ctorParameters = function () { return [
        { type: IgxGridAPIService, },
        { type: ElementRef, },
        { type: NgZone, },
        { type: undefined, decorators: [{ type: Inject, args: [DOCUMENT,] },] },
        { type: ChangeDetectorRef, },
        { type: ComponentFactoryResolver, },
        { type: ViewContainerRef, },
    ]; };
    IgxGridComponent.propDecorators = {
        "data": [{ type: Input },],
        "autoGenerate": [{ type: Input },],
        "id": [{ type: Input },],
        "filteringLogic": [{ type: Input },],
        "filteringExpressions": [{ type: Input },],
        "paging": [{ type: Input },],
        "page": [{ type: Input },],
        "perPage": [{ type: Input },],
        "paginationTemplate": [{ type: Input },],
        "height": [{ type: HostBinding, args: ["style.height",] }, { type: Input },],
        "width": [{ type: HostBinding, args: ["style.width",] }, { type: Input },],
        "evenRowCSS": [{ type: Input },],
        "oddRowCSS": [{ type: Input },],
        "rowHeight": [{ type: Input },],
        "onSelection": [{ type: Output },],
        "onColumnPinning": [{ type: Output },],
        "onEditDone": [{ type: Output },],
        "onColumnInit": [{ type: Output },],
        "onSortingDone": [{ type: Output },],
        "onFilteringDone": [{ type: Output },],
        "onPagingDone": [{ type: Output },],
        "onRowAdded": [{ type: Output },],
        "onRowDeleted": [{ type: Output },],
        "columnList": [{ type: ContentChildren, args: [IgxColumnComponent, { read: IgxColumnComponent },] },],
        "rowList": [{ type: ViewChildren, args: [IgxGridRowComponent, { read: IgxGridRowComponent },] },],
        "parentVirtDir": [{ type: ViewChild, args: ["scrollContainer", { read: IgxForOfDirective },] },],
        "verticalScrollContainer": [{ type: ViewChild, args: ["verticalScrollContainer", { read: IgxForOfDirective },] },],
        "scr": [{ type: ViewChild, args: ["scr", { read: ElementRef },] },],
        "paginator": [{ type: ViewChild, args: ["paginator", { read: ElementRef },] },],
        "headerContainer": [{ type: ViewChild, args: ["headerContainer", { read: IgxForOfDirective },] },],
        "theadRow": [{ type: ViewChild, args: ["theadRow",] },],
        "tfoot": [{ type: ViewChild, args: ["tfoot",] },],
        "tabindex": [{ type: HostBinding, args: ["attr.tabindex",] },],
        "hostClass": [{ type: HostBinding, args: ["attr.class",] },],
        "hostRole": [{ type: HostBinding, args: ["attr.role",] },],
        "sortingExpressions": [{ type: Input },],
    };
    return IgxGridComponent;
}());
export { IgxGridComponent };
//# sourceMappingURL=grid.component.js.map