import { EventEmitter, NgZone } from "@angular/core";
/**
 * IgxSnackbar provides feedback about an operation by showing brief message at the bottom of the screen on mobile
 * and lower left on larger devices. IgxSnackbar will appear above all
 * other elements on screen and only one can be displayed at a time.
 * ```
 * <igx-snackbar (event output bindings) [input bindings]>
 * </igx-snackbar>
 * ```
 */
export declare class IgxSnackbarComponent {
    private zone;
    /**
     * The message that will be shown message by the IgxSnackbar component
     * @type {string}
     */
    message: string;
    /**
     * The IgxSnackbar component visual state state
     * @type {boolean}
     */
    isVisible: boolean;
    /**
     * Sets if the IgxSnackbar component will be hidden after shown
     * Default value is true
     * @type {number}
     */
    autoHide: boolean;
    /**
     * The duration of time span in ms which the IgxSnackbar component will be visible
     * after it is being shown.
     * Default value is 4000
     * @type {number}
     */
    displayTime: number;
    /**
     * The text of the IgxSnackbar component action
     * @type {string}
     */
    actionText?: string;
    /**
     * The event that will be thrown when the action is executed,
     * provides reference to the IgxSnackbar component as argument
     * @type {EventEmitter}
     */
    onAction: EventEmitter<{}>;
    /**
     * The event that will be thrown when the snackbar animation starts
     * @type {EventEmitter<AnimationTransitionEvent>}
     */
    animationStarted: EventEmitter<any>;
    /**
     * The event that will be thrown when the snackbar animation ends
     * @type {EventEmitter<AnimationTransitionEvent>}
     */
    animationDone: EventEmitter<any>;
    private timeoutId;
    constructor(zone: NgZone);
    /**
     * Shows the IgxSnackbar component and hides it after some time span
     * if autoHide is enabled
     */
    show(): void;
    /**
     * Hides the IgxSnackbar component
     */
    hide(): void;
    triggerAction(): void;
    snackbarAnimationStarted(evt?: any): void;
    snackbarAnimationDone(evt?: any): void;
}
export declare class IgxSnackbarModule {
}
